<?php

namespace Drupal\murmurations\Form;

use Drupal\Core\Url;
use Drupal\Core\Link;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\leaflet\LeafletService;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\murmurations\MurmurationsPluginManager;

/**
 * Inject Leaflet service
 */
class FilterFormMap extends FilterForm {

  const MAP_WIDTH = '400px';

  protected $leaflet;
  protected $pathResolver;

 /**
  *
  * @param type $user_data
  * @param type $request
  * @param type $route_match
  * @param MurmurationsPluginManager $murm_plugin_manager
  * @param Drupal\leaflet\LeafletService $leaflet_service
  * @param Drupal\Core\Extension\ExtensionPathResolver $path_resolver
  */
  public function __construct($user_data, $request, $route_match, MurmurationsPluginManager $murm_plugin_manager, LeafletService $leaflet_service, ExtensionPathResolver $path_resolver) {
    parent::__construct($user_data, $request, $route_match, $murm_plugin_manager);
    $this->leaflet = $leaflet_service;
    $this->pathResolver = $path_resolver;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('user.data'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('current_route_match'),
      $container->get('plugin.manager.murmurations'),
      $container->get('leaflet.service'),
      $container->get('extension.path.resolver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'murmurations_filter_form_map';
  }

  /**
   * Show the results in a map.
   *
   * @param array $results
   * @param int $number_of_results
   * @return array
   */
  public function showResults(array $results, int $number_of_results) : array {
    $features = [];
    foreach ($results as $result) {
      if (!isset($result->geolocation)) {
        continue;
      }
      $dest_url = Url::fromUri($result->primary_url);
      $popup = Link::fromTextAndUrl($result->title, $dest_url)->toString() . '<br />'.
        text_summary($result->description, filter_fallback_format(), 150).'<br />'.
        $result->tags;
      $features[] = [
        'type' => 'point',
        'lat' => $result->geolocation->lat,
        'lon' => $result->geolocation->lon,
        'icon' => [
          'iconUrl' => '/'.$this->pathResolver->getPath('module', 'murmurations') . '/images/bluepin.png',
          //'icon_size' => ['x' => 32, 'y' => 32],
          'icon_anchor' => ['x' => 24, 'y' => 32],//not supported yet in leaflet module Beta1
          'popup_anchor' => ['x' => 16, 'y' => 0],
        ],
        'popup' => $popup,
        'leaflet_id' => $result->primary_url
      ];
    }
    if ($features) {
      return $this->leaflet->leafletRenderMap(
        leaflet_leaflet_map_info()['OSM Mapnik'],
        $features,
        SELF::MAP_WIDTH
      );
    }
    return [];
  }

}
