<?php

namespace Drupal\murmurations\Plugin\Murmurations;

use Drupal\Core\Entity\EntityInterface;
use Drupal\murmurations\Plugin\Murmurations\PluginBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\murmurations\MurmurationsPluginMultipleInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Entity\EntityFieldManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @todo Inject entityTypeManager.
 */
abstract class PluginMultipleBase extends PluginBase implements MurmurationsPluginMultipleInterface {

  /**
   * @var Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * @var Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * @var EntityInterface
   */
  protected $entity;

  /**
   *
   * @param $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param $config_factory
   * @param EntityTypeManager $entity_type_manager
   * @param EntityFieldManager $entity_field_manager
   */
  function __construct($configuration, $plugin_id, $plugin_definition, $config_factory, EntityTypeManager $entity_type_manager, EntityFieldManager $entity_field_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $config_factory);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritDoc}
   */
  function needed() : string {
    return $this->getEntityIds() ? '' : t('No itesm are eligible to be published');
  }

  /**
   * {@inheritDoc}
   */
  function setEntity(EntityInterface $entity) : self {
    $this->entity = $entity;
    return $this;
  }

  /**
   * {@inheritDoc}
   */
  function getPublishPath() : string {
    // Replace the parameter in the publish path
    return preg_replace('/\{[a-z]+\}/', $this->entity->id(), $this->getPluginDefinition()['profile_path']);
  }

  /**
   * {@inheritDoc}
   */
  public function getProfilePaths() : array {
    $paths=[];
    foreach ($this->getEntityIds() as $id) {
      $entity_type_id = $this->getPluginDefinition()['entity_type'];
      $entity = $this->entityTypeManager->getStorage($entity_type_id)->load($id);
      $paths[] = $this->setEntity($entity)->getPublishPath();
    }
    return $paths;
  }

  /**
   * {@inheritDoc}
   */
  abstract function getEntityIds() : array;

  /**
   *
   * @return array
   *   Find the geofield and if not, find the geofield of the owner, and if not, get the site geofield
   */
  function getCoords() : array {
    if ($field = $this->geolocateEntityField($this->entity)) {
      $point = ['lat' => $field_items->lat, 'lon' => $field_items->lon];
    }
    else {
      // we apply displacement to this to prevent all entities being rendered on the same point.
      $point = parent::getCoords();
    }
    return displace_point($point, $this->configuration['displacement']);
 }

  private static function geolocateEntityField(ContentEntityInterface $entity) : FieldItems {
    $fields = $this->entityFieldManager
      ->getFieldDefinitions($this->entity->getEntityTypeId(), $this->entity->bundle());
    foreach ($fields as $field_name => $field_info) {
      if ($field_info->getType() == 'geofield') {
        $field_items = $this->entity->{$field_name};
        if ($field_items->lat) {
          return $field_items;
        }
      }
    }
    if ($this->entity instanceof \Drupal\user\EntityOwnerInterface) {
      return $this->geolocateEntity($this->entity->getOwner());
    }
  }

}
